#ifndef lint
static char	*rcsid = "@(#)$Header: clientlib.c,v 1.16 91/01/12 15:12:37 sob Exp $";
#endif
/*
 * Clientlib modified for lib support only.  News is read locally, no server
 * is used.  Modified by for NetNews/CD support.
 * 4-92 - Tom Theel
 *        Sterling Software
 *        cddev@imd.sterling.com or tom@imd.sterling.com
 *
 * Based on:
 *
 * This software is Copyright 1991 by Stan Barber. 
 *
 * Permission is hereby granted to copy, reproduce, redistribute or otherwise
 * use this software as long as: there is no monetary profit gained
 * specifically from the use or reproduction or this software, it is not
 * sold, rented, traded or otherwise marketed, and this copyright notice is
 * included prominently in any copy made. 
 *
 * The author make no claims as to the fitness or correctness of this software
 * for any use whatsoever, and it is provided as is. Any use of this software
 * is at the user's own risk. 
 *
 */
/*
 * NNTP client routines.
 */

/*
 * Include configuration parameters only if we're made in the nntp tree.
 */

#include <stdio.h>
#include <sys/types.h>
#include <ctype.h>
#include "common.h"
#include "conf.h"
#include "spooldir.h"

extern struct spooldir spdir;
extern struct newscontrol newscont;


static struct cmdent {
        char    *cmd_name;
        int     authreq;        /* 0=none,1=userpass */
        int     (*cmd_fctn)();
} cmdtbl[] = {
        "article",      0,      ahbs,
        "body",         0,      ahbs,
        "group",        0,      group,
        "head",         0,      ahbs,
        "help",         0,      help,
        "ihave",        1,      ihave,
        "last",         0,      nextlast,
        "list",         0,      list,
        "newgroups",    0,      newgroups,
        "newnews",      0,      newnews,
        "next",         0,      nextlast,
        "post",         1,      post,
        "slave",        0,      slave,
        "spooldir",     0,      spldir,
        "stat",         0,      ahbs,
#ifdef XHDR
        "xhdr",         0,      xhdr,
#endif XHDR
#ifdef XTHREAD
        "xthread",      0,      xthread,
#endif
};
#define NUMCMDS (sizeof(cmdtbl) / sizeof(struct cmdent))

/*
 * getserverbyfile	Not really needed for lib support.
 *                      Just here for current clientlib compatibility.
 *
 *	Parameters:	not used for this version
 *
 *	Returns:	Pointer to static data area containing "lib". 
 *
 *	Side effects:	None.
 */

char *
getserverbyfile(file)
char	*file;
{
    static char	buf[] = {'l','i','b'};

    return (buf);
}


/*
 * server_init  initialize clientlib-level software. 
 *
 *	Parameters:	not really used for this version
 *
 *	Returns:        Server Ready, No Posting Allowed code.	
 *
 *	Side effects:   None	
 */

server_init(machine)
char	*machine;
{
    static first_time = 1;

    if (first_time) {
        spdir.transtype = DEFAULT_TTYPE;
        (void) strcpy(spdir.dirpath, SPOOLDIR);
        (void) strcpy(spdir.name, DEFAULT_SPOOL);
        (void) strcpy(spdir.libpath, LIBDIR);
	first_time = 0;
    }
    
    if (spdir.transtype == TYPE_ISO) {
	/* Need to determine the iso extension that is on files without
	 * extensions (such as "active" or an article "1234").
	 */
        (void) sprintf(newscont.active, "%s/%s", spdir.libpath, 
			ACTIVE_FILE_ISO);
	if (determine_iso_extension(newscont.active, iso_extension) != 1) {
#ifndef DONT_ALLOW_PRINTF
	    printf("Failed to determine iso extension used by cdrom driver\n");
#endif
	    first_time = 1;
	    return(ERR_GOODBYE);
	}
        (void) sprintf(newscont.active, "%s/%s%s", spdir.libpath, 
			ACTIVE_FILE_ISO, iso_extension);
        (void) sprintf(newscont.newsgroups, "%s/%s%s", spdir.libpath, 
			NEWSGROUPS_FILE_ISO, iso_extension);
	/* active.tim has an extension, so iso_extension is unnecessary */
        (void) sprintf(newscont.active_times, "%s/%s", spdir.libpath,
			ACTIVE_TIMES_FILE_ISO);
    } else {
        (void) sprintf(newscont.active, "%s/%s", spdir.libpath, ACTIVE_FILE);
        (void) sprintf(newscont.newsgroups, "%s/%s", spdir.libpath, 
                   NEWSGROUPS_FILE);
        (void) sprintf(newscont.active_times, "%s/%s", spdir.libpath,
                   ACTIVE_TIMES_FILE);
    }
    (void) sprintf(newscont.distributions, "%s/%s", spdir.libpath, 
                   DISTRIBUTIONS_FILE);
    (void) sprintf(newscont.history, "%s/%s", spdir.libpath, HISTORY_FILE);
    threaddirpath = spdir.dirpath;

    state_cb.state = NULL_STATE;
    state_cb.help_index = 0;
    ingroup = 0;		/* we are not in a group currently */
    num_groups = 0;
    num_groups = read_groups(); /* Read in the active file */
    return (OK_NOPOST);  /* Server Ready, No Posting Allowed */
}


/*
 * handle_server_response
 *
 *	Print some informative messages based on the server's initial
 *	response code.  This is here so inews, rn, etc. can share
 *	the code.  Note: not really used/tested for NetNews/CD clientlib
 *      version.
 *
 *	Parameters:	"response" is the response code which the
 *			server sent us, presumably from "server_init",
 *			above.
 *			"nntpserver" is the news server we got the
 *			response code from.
 *
 *	Returns:	-1 if the error is fatal (and we should exit).
 *			0 otherwise.
 *
 *	Side effects:	None.
 */

handle_server_response(response, nntpserver)
int	response;
char	*nntpserver;
{
    switch (response) {
	case OK_NOPOST:		/* fall through */
#ifndef DONT_ALLOW_PRINTF
    	    (void) printf(
	"NOTE: This machine does not have permission to post articles.\n");
	    (void) printf(
	"      Please don't waste your time trying.\n\n");
#endif

	case OK_CANPOST:
	    return (0);
	    break;

	case ERR_ACCESS:
#ifndef DONT_ALLOW_PRINTF
	    (void) printf(
   "This machine does not have permission to use the %s news server.\n",
		nntpserver);
#endif
	    return (-1);
	    break;

	default:
#ifndef DONT_ALLOW_PRINTF
	    (void) printf("Unexpected response code from %s news server: %d\n",
			nntpserver, response);
#endif
	    return (-1);
	    break;
    }
	/*NOTREACHED*/
}


/*
 * put_server -- process string which would have gone to a real server in 
 *               a normal environment. 
 *
 *	Parameters:	"line" is the string to process.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:   None	
 *
 */

void
put_server(line)
char *line;
{
    static char **argp = NULL;
    int         argnum, i;

    /* Null command */
    if ((argnum = parsit(line, &argp)) == 0)
        return;

#ifdef DEBUG
printf("put_server [%s]\n", line);
#endif
    /* This is a single response line command */
    state_cb.state = SINGLE_RESP_STATE;

    /* a motion to adjourn is always in order */
    if (!strcasecmp(argp[0], "quit"))
        (void) sprintf(response_text, "%d closing connection.  Goodbye.\r\n",
                OK_GOODBYE);
    else {
        for (i = 0;i < NUMCMDS; ++i)
            if (!strcasecmp(cmdtbl[i].cmd_name, argp[0]))
                break;

        if (i < NUMCMDS)
            (*cmdtbl[i].cmd_fctn)(argnum, argp);
        else
            (void) sprintf(response_text,
                           "%d %s command unrecognized.\r\n",
                           ERR_COMMAND, argp[0]);
    }
}


/*
 * get_server -- get a line of text from state() routine.
 *
 *	Parameters:	"string" has the buffer space for the
 *			line received.
 *			"size" is the size of the buffer.
 *
 *	Returns:	0
 *
 *	Side effects:	Current state used in state() may change.
 *                      Changes contents of "string".
 */

get_server(string, size)
char	*string;
int	size;
{
    (void) strncpy(string, state(), size);
    return(0);
}


static	long	rawbytes = -1;	/* bytes remaining to be transfered */
FILE *xthreadfp;  /* opened in xthread() */

/*
 * rawcheck_server -- get a line of text via get_server, interpreting
 * it as a status message for a raw (binary) command.  Call this once
 * before calling rawget_server() for the actual data transfer.
 *
 *	Parameters:	"string" has the buffer space for the
 *			line received.
 *			"size" is the size of the buffer.
 *
 *	Returns:	-1 on error, otherwise the length of the raw data.
 *
 *	Side effects:	Changes contents of "string".
 */
long
rawcheck_server(string, size)
char	*string;
int	size;
{
    /* try to get the status line and the status code */
    if (get_server(string, size) || *string != CHAR_OK)
        return rawbytes = -1;
#ifdef RAW_SERVER_DEBUG
    printf("clientlib rawcheck_server() - string [%s]\n", string);
#endif
    /* try to get the number of bytes being transfered */
    if (sscanf(string, "%*d%ld", &rawbytes) != 1)
	return rawbytes = -1;
    return rawbytes;
}


/*
 * rawget_server -- read data from an open file in raw format.  This call must
 * follow an appropriate put_server command and a rawcheck_server call.
 *
 *	Parameters:	"buf" is the buffer for the data to receive.
 *			"n" is the size of the buffer.
 *
 *	Returns:	0 on EOF, otherwise the length of the read.
 *
 *	Side effects:	Changes contents of "buf".
 */
long
rawget_server(buf, n)
void	*buf;
long	n;
{
    /* if no bytes to read, then just return EOF */
    if (rawbytes < 0)
        return 0;

    /* try to read some data from the server */
    if (rawbytes) {
        n = fread(buf, 1, n > rawbytes ? rawbytes : n, xthreadfp);
        rawbytes -= n;
    } else { /* we've already read the entire file */
        n = 0;
	rawbytes = -1;
        fclose(xthreadfp);
    }

#ifdef RAW_SERVER_DEBUG
    printf("clientlib rawget_server() - returning %d\n", n);
#endif
    return n;
}


/*
 * close_server -- do a "quit" command. 
 *
 *	Parameters:	None.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:   You can't use "put_server" or "get_server"
 *			after this routine is called.
 */

void
close_server()
{
    put_server("QUIT");
}
