/*
** @(#)spooldir.h	1.1 4/24/92
**
** This file is used to provide the clientlib and nntpserver required defines
** and structures to support the multiple spool directory extensions.
**
*/


/*
** ================
** SPOOLDIR [name]
** 
** With no specified name parameter, SPOOLDIR requests return a list of
** names/aliases for the news spool directories which are available
** for reading news from.  The spool directory named "news" will be
** used to indicate the default standard live traffic news reading.
** 
** When the name of a spool directory is specified, the NNTP server
** switches to supplying news from the named spool directory.  If the
** name of the spool directory is not "news" then it is considered
** an archive spool directory.  The only operation allowed on an archive
** news spool directory is reading.  Operations such as post, and transfer
** do not apply. 
** 
** When the SPOOLDIR command is used to switch directories, newsreaders 
** should assure that internal references to articles numbers and newsgroups 
** in the previous spool directory are reinitialized to reflect the change 
** in spool directory information.
** ================
** 
** Implementation Notes:
** 
** The nntp server will need to determine where to read news from.  This
** means that the spool directories must be registered in a configuration
** file, /usr/lib/news/spooldir by default.   If there is no spooldir file
** on the system when the server starts up it creates one with
** 
** #
** # SpoolDir  Mount     Library Directory   Spool Directory           Newsgroup
** #  Name     Point         Path                Path                Translation
** #
** news        NA    /usr/lib/news         /usr/spool/news           unix    
** 
** as the default information.  A sample of a spooldir file for a system
** that had 2 CD-ROMs online along with the normal news directories may
** look like:
** 
** #
** # SpoolDir  Mount    Library Directory   Spool Directory          Newsgroup
** #  Name     Point         Path                Path               Translation
** #
** news        NA       /usr/lib/news         /usr/spool/news           unix    
** Jan1-20     /cdrom   /cdrom/usr/lib/news   /cdrom/usr/spool/news     unix    
** May15-30    /cdrom   /cdrom1/usr/lib/news  /cdrom1/usr/spool/news    ISO
** 
** 
** The Newsgroup Translation is an indication if the nntp server should
** use a name translation lookup. The name translation lookup is needed
** for converting ISO restrictive names (8.3 DOS naming) into standard
** newsgroup names.  If the name translation is unix then it does no
** lookups and converts the '.'s to '/'s and uses it relative to the
** spool directory path.  If ISO is specified then the server does
** a newsgroup name lookup and returns an actual path that nntpd will
** need to locate the article.
** 
** The server will supply news from the spooldir listed as "news" as the
** default.  If there is no spooldir named "news" then the first entry
** in the spooldir file is considered the default spool directory. 
** 
*/ 

#include <sys/param.h>

struct spooldir {
	char name[15];
	char mntpoint[MAXPATHLEN];
	char libpath[MAXPATHLEN];
	char dirpath[MAXPATHLEN];
	char descrip[81];
	int transtype;
};

struct newscontrol {
	char active[MAXPATHLEN];
	char newsgroups[MAXPATHLEN];
	char distributions[MAXPATHLEN];
	char history[MAXPATHLEN];
	char nntp_access[MAXPATHLEN];
	char active_times[MAXPATHLEN];
#ifdef AUTH
	char passfile[MAXPATHLEN];
#endif
};
	

#define TYPE_UNIX 0
#define TYPE_ISO  1


#define SPOOLDIRS_FILE    "/cdrom/lib/spooldir.cf" /* for cdrn executable */
#define PATH_TRANS_FILE   "pathtran"

#define DEFAULT_SPOOL "whatever"
#define DEFAULT_TTYPE TYPE_UNIX
