/* Modified by Sterling Software for use with clientlib interface to
 * read NetNews/CDs.
 * 3/23/92
 */
/* This file (and only this file - not the entire nntp distribution) is
 * hereby placed in the public domain.  Use it as you see fit, but if you
 * manage to find some wonderful use for this code elsewhere, it would be
 * nice to receive mention for it.
 *
 * - Tim Iverson
 *   iverson@xstor.com -/- uunet!xstor!iverson
 *   3/28/91
 *   modified by Wayne Davison (davison@borland.com) to work with trn 2.0
 *   10/6/91
 *   modified by Sterling Software (cdnews@imd.sterling.com) for use with a
 *   clientlib interface to read NetNews/CDs.
 *   3/23/92
 */

#include "common.h"
#include "spooldir.h"

extern struct spooldir spdir;

#ifdef XTHREAD

# ifdef __GNUC__
#  define alloca __builtin_alloca
# endif

FILE	*xthreadfp;

/* Usage: XTHREAD [DBINIT|THREAD]
 *
 * DBINIT	dump the contents of the db.init file to stdout
 * THREAD	dump the contents of the thread file for the current
 *		newsgroup to stdout (default if no argument).
 *
 * N.B. These two files are not ascii and no attempt is made at converting
 *	native byte size to any type of standard whatsoever.  This'll have
 *	to be fixed if this command is to be integrated into the protocol.
 *
 * This command is not documented in rfc977.
 */

xthread(argc, argv)
int	argc;
char	*argv[];
{
    struct stat		s;
    char		*file, *what;

    state_cb.state = XTHREAD_STATE;
    /* "parse" the argument */
    if (argc == 2 && !strcasecmp(argv[1], "dbinit")) {
	char *cp, *thread_filename();

	file = thread_filename("*******");
        if (spdir.transtype == TYPE_ISO)
	    what = "dbi.nit";
        else
	    what = "db.init";
	for (cp = file; *cp != '*'; cp++)
			;
	strcpy(cp, what);
    } else if (argc == 1 || argc == 2 && !strcasecmp(argv[1], "thread")) {
	if (!threadfile) {
	    (void) sprintf(response_text, 
                           "%d You are not currently in a newsgroup.\r\n",
				ERR_NCING);
	    return;
	}
	file = threadfile;
	what = "thread";
    } else {
	(void) sprintf(response_text, 
                       "%d Usage: XTHREAD [DBINIT|THREAD]\r\n", ERR_CMDSYN);
	return;
    }

#ifdef PATH_DEBUG
    printf("xthread trying to open [%s]\n", file);
#endif
    /* try to open the file to be transfered */
    if (!(xthreadfp = fopen(file, "r"))) {
	(void) sprintf(response_text, 
                       "%d %s file is not available.\r\n", ERR_FAULT, what);
	return;
    }

    /* tell 'em how much binary data is coming down the pike */
    fstat(fileno(xthreadfp), &s);
    (void) sprintf(response_text, 
                   "%d %u bytes of %s file follows verbatim (binary!)\r\n",
		OK_BIN, s.st_size, what);
    return;
}


/* Change a newsgroup name into the name of the thread data file.  We
** subsitute any '.'s in the group name into '/'s prepend the path, 
** and append the '/THREAD' (or '/.thread') on to the end.
*/
char *
thread_filename(group)
char *group;
{
    static char name_buff[MAXPATHLEN];
    char group_buff[MAXPATHLEN];

    if (group[0] == '*')
        strcpy(group_buff, group);
    else
        (void) findpath(group, group_buff, spdir.libpath, spdir.transtype);

#ifdef PATH_DEBUG
    printf("thread_filename findpath for [%s] is [%s]\n", group, group_buff);
#endif
    if (spdir.transtype == TYPE_ISO)
        sprintf(name_buff, "%s/%s%s%s", threaddirpath, group_buff, ISO_THREAD,
		iso_extension);
    else
        sprintf(name_buff, "%s/%s%s", threaddirpath, group_buff, UNIX_THREAD);

#ifdef PATH_DEBUG
    printf("thread_filename returning [%s]\n", name_buff);
#endif
    return name_buff;
}
#endif /* not XTHREAD */
